use strict;
use Win32::SerialPort qw(:STAT 0.19);

local $| = undef;

# INIT
my $port = new Win32::SerialPort('COM1', 1) or die "Can't open port: $^E\n";
$port->handshake('none');
$port->baudrate(2400);
$port->databits(8);
$port->stopbits(2);
$port->parity('none');
$port->write_settings() or die "Can't establish connection: $^E\n";


# CONTROL
foreach (sense(1)) {
    print $_ ? 'X' : '-';
}
print "\n";

engine(46, 0, 1);
functions(46, 0, 1, 1, 0);
invert(46, 1);
engine(46, 6, 1);
sleep(2);
engine(46, 0, 1);
sleep(4);
invert(46, 1);
engine(46, 6, 1);
sleep(2);
engine(46, 0, 1);
sleep(4);
functions(46, 0, 0, 0, 0);
engine(46, 0, 0);

foreach (sense(1)) {
    print $_ ? 'X' : '-';
}
print "\n";

# CLOSE
$port->close();




# HELP SUBROUTINES

sub boolsplit($) {
    my @output;
    foreach my $byte (split(//, $_[0])) {
        foreach my $bit (0..7) {
            push(@output, (ord($byte) & (1 << $bit)) > 0);
        }
    }
    return @output;
}


# TRAIN CONTROL SUBROUTINES

sub stop() { transmit(97) }

sub start() { transmit(96) }

sub engine($$$) { # engine id, speed 0..14, function-p
    warn "engine id ($_[0]) out of range\n" if $_[0] < 1 or $_[0] > 80;
    warn "speed ($_[1]) out of range\n" if $_[1] < 0 or $_[1] > 14;
    transmit($_[1] + ($_[2] ? 16 : 0), $_[0]);
}

sub invert($$) { # engine id, function-p
    warn "engine id ($_[0]) out of range" if $_[0] < 1 or $_[0] > 80;
    # for some reason, you need to send an engine stop command first
    engine($_[0], 0, $_[1]);
    transmit(15 + ($_[1] ? 16 : 0), $_[0]);
    engine($_[0], 0, $_[1]);
}

sub functions($$$$$) { # engine id, f1, f2, f3, f4
    warn "engine id ($_[0]) out of range\n" if $_[0] < 1 or $_[0] > 80;
    transmit(64 + ($_[1] ? 1 : 0) + ($_[2] ? 2 : 0) + ($_[3] ? 4 : 0) + ($_[4] ? 8 : 0), $_[0]);
}

sub switch($$) { # switch id, direction (1 is branch/outer/red, 0 is straight/inner/green)
    warn "switch id ($_[0]) out of range\n" if $_[0] < 1 or $_[0] > 256;
    transmit($_[1] ? 34 : 33, $_[0] == 256 ? 0 : $_[0]);
    transmit(32);
}

sub sense($) { # sensor id
    warn "sensor id ($_[0]) out of range\n" if $_[0] < 1 or $_[0] > 31;
    transmit(192 + $_[0]);
    return boolsplit receive();
}

sub senses($) { # number of sensors to read
    warn "sensor count ($_[0]) out of range\n" if $_[0] < 1 or $_[0] > 31;
    transmit(128 + $_[0]);
    return boolsplit receive();
}


# PORT CONTROL SUBROUTINES

sub transmit {
    print 'transmitting: ';
    foreach (@_) {
        printf('%02x (%03d) ', $_, $_);
        $port->transmit_char($_);
    }
    select(undef, undef, undef, 0.15);
    print "\n";
}

sub receive {
    print 'receiving: ';
    my $data = $port->input;
    foreach (split(//, $data)) {
        printf('%08b ', ord($_));
    }
    print "\n";
    return $data;
}